// -*- C++ -*-
#ifndef RIVET_FromElectroweakDecay_HH
#define RIVET_FromElectroweakDecay_HH

#include "Rivet/Tools/Logging.hh"
#include "Rivet/Rivet.hh"
#include "Rivet/Particle.hh"
#include "Rivet/Event.hh"
#include "Rivet/Projection.hh"
#include "Rivet/Projections/FinalState.hh"

namespace Rivet {


  /// @brief Project only charged final state particles.
  class FromElectroweakDecay : public FinalState {
  public:

    /// @name Constructors
    //@{
    // Final State
    FromElectroweakDecay(const FinalState& fsp);

    /// Single eta-range constructor.
    FromElectroweakDecay(double mineta = -MAXDOUBLE,
			 double maxeta =  MAXDOUBLE,
			 double minpt  =  0.0*GeV);

    /// A constructor which allows to specify multiple eta ranges
    /// and the min \f$ p_T \f$.
    FromElectroweakDecay(const vector<pair<double, double> >& etaRanges,
			  double minpt = 0.0*GeV);

    /// Clone on the heap.
    virtual const Projection* clone() const {
      return new FromElectroweakDecay(*this);
    }
    //@}

    void acceptTauDecays(bool b = true) { _acceptTauDecays = b; } 

    bool electroweakFilter(const Particle& p) const;

  protected:

    /// Apply the projection on the supplied event.
    void project(const Event& e);

    /// Compare projections.
    int compare(const Projection& p) const;

  private:
    bool _acceptTauDecays;

  };

}


#endif
