// -*- C++ -*-
#include "Rivet/Projections/FromElectroweakDecay.hh"

namespace Rivet {


  FromElectroweakDecay::FromElectroweakDecay(const FinalState& fsp):
    _acceptTauDecays(false) {
    setName("FromElectroweakDecay");
    addProjection(fsp, "FS");
  }


  FromElectroweakDecay::FromElectroweakDecay(double mineta, double maxeta, double minpt):
    _acceptTauDecays(false) {
    setName("FromElectroweakDecay");
    addProjection(FinalState(mineta, maxeta, minpt), "FS");
  }


  FromElectroweakDecay::FromElectroweakDecay(const vector<pair<double, double> >& etaRanges,
					       double minpt):
    _acceptTauDecays(false) {
    setName("FromElectroweakDecay");
    addProjection(FinalState(etaRanges, minpt), "FS");
  }


  int FromElectroweakDecay::compare(const Projection& p) const {
    return mkNamedPCmp(p, "FS");
  }


  bool FromElectroweakDecay::electroweakFilter(const Particle& p) const {

    /// @todo Shouldn't a const vertex be being returned? Ah, HepMC...
    GenVertex* prodVtx = p.genParticle()->production_vertex();
    if (prodVtx == NULL) return false; // orphaned lepton

    foreach (const GenParticle* ancestor, Rivet::particles(prodVtx, HepMC::ancestors)) {
      const PdgId pid = ancestor->pdg_id();
      if(ancestor->status() != 2) continue;
      if(PID::isHadron(pid)) return false;
      if(!_acceptTauDecays && abs(pid) == PID::TAU) return false;
    }
    return true;
  }


  void FromElectroweakDecay::project(const Event& e) {
    const FinalState& fs = applyProjection<FinalState>(e, "FS");
    _theParticles.clear();
    
    const Particles& particles = fs.particles();
    foreach (const Particle& p, particles) {
      if(!electroweakFilter(p)) continue;
      _theParticles.push_back(p);
    }

    MSG_DEBUG("Number of final state particles not from hadron decays = " << _theParticles.size());
    if (getLog().isActive(Log::TRACE)) {
      for (vector<Particle>::iterator p = _theParticles.begin(); p != _theParticles.end(); ++p) {
        MSG_TRACE("Selected: " << p->pdgId() << ", charge = " << PID::threeCharge(p->pdgId())/3.0);
      }
    }
  }


}
